@@ -139,6 +139,9 @@ atk_role_get_type (void)
       { ATK_ROLE_MATH, "ATK_ROLE_MATH", "math" },
       { ATK_ROLE_RATING, "ATK_ROLE_RATING", "rating" },
       { ATK_ROLE_TIMER, "ATK_ROLE_TIMER", "timer" },
+      { ATK_ROLE_DESCRIPTION_LIST, "ATK_ROLE_DESCRIPTION_LIST", "description-list" },
+      { ATK_ROLE_DESCRIPTION_TERM, "ATK_ROLE_DESCRIPTION_TERM", "description-term" },
+      { ATK_ROLE_DESCRIPTION_VALUE, "ATK_ROLE_DESCRIPTION_VALUE", "description-value" },
       { ATK_ROLE_LAST_DEFINED, "ATK_ROLE_LAST_DEFINED", "last-defined" },
       { 0, NULL, NULL }
     };
